# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.browser.generic

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for browser.generic tests """
    LOAD_MODULES = (('_generic', 'svnmailer.browser.generic'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_browser_base(),
        ]


#### Test Cases                                     ####
########################################################
class GeneratorTestCase(TestBase):
    """ svnmailer.browser.generic.Generator """
    PREFIX = "Generator"

    def test00general(self):
        """ Generator is a public _base.BaseGenerator """
        expected = self.mock['svnmailer.browser._base'].BaseGenerator
        result = self._generic.Generator

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._generic),
            "Is not public")

    def test01createTemplate(self):
        """ ._createTemplate returns a Template """
        inst = self._generic.Generator()
        config = base.Store(
            revision_url      = 'url:revision',
            diff_delete_url   = 'url:delete',
            diff_copy_url     = 'url:copy',
            diff_add_url      = 'url:add',
            diff_modify_url   = 'url:modify',
        )
        self.mock.result.append(inst._createTemplate('foo', config)._templates)

        self.expect({
            'deleted' : 'url:delete',
            'copied'  : 'url:copy',
            'added'   : 'url:add',
            'modified': 'url:modify',
            'revision': 'url:revision'
        })

    def test02createTemplate(self):
        """ ._createTemplate passes AttibuteError """
        self.mock.error = 'browser.template.attr'
        inst = self._generic.Generator()

        self.failUnlessRaises(AttributeError,
            inst._createTemplate, 'foo', 'bar')

_tests.append(GeneratorTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
