# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0321,E0201,E0213,W0131,F0203,C0103,W0232,R0914,W0612
# pylint: disable-msg=W0703
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.main

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
_tests = []


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.main """

    def testImport(self):
        """ from svnmailer import main """
        try:
            from svnmailer import main
        except Exception, exc:
            self.fail("svnmailer.main could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class MainTestCase(unittest.TestCase):
    """ svnmailer.main.Main """

    def setUp(self):
        """ Build test environment """
        from svnmailer import main

        self._main = main

    def test00runMethod(self):
        """ run() manages the repos and runs the notifiers for every groupset """
        supplied = [
            [ ['group1', 'group2'], ['change1'           ] ],
            [ ['group3'          ], ['change2', 'change3'] ],
            [ ['group4', 'group5'], ['change4', 'change5'] ],
        ]
        expected = ["foo"] + supplied + ["bar"]
        result = []

        class myMain(self._main.Main):
            def __init__(this): pass
            def _openRepository(this): result.append("foo")
            def _closeRepository(this): result.append("bar")
            def _getGroupSets(this): return supplied[:]
            def _getNotifierSelector(this):
                class selector(object):
                    def selectNotifiers(that, groupset):
                        result.append(groupset)
                        return []
                return selector()

        myMain().run()
        self.failUnlessEqual(
            expected, result,
            "Notifiers were not run properly. Unexpected result is: %r" %
            result
        )

    def test01getGroupsByChange(self):
        """ getGroupsByChange returns the proper lists """
        import re
        config = [Container(
            _name = name, ignore_if_other_matches = ignore,
            for_repos = repos and re.compile(repos) or None,
            for_paths = paths and re.compile(paths) or None,
            exclude_paths = excl and re.compile(excl) or None,
            _sub_ = lambda x, y: None,
        )
        # put new groups here:
        # ---------------------
        for  name, repos,   paths,   excl,        ignore in (
            ("g1", None,    "/",     None,        True),
            ("g2", None,    "/path", None,        False),
            ("g3", None,    "/path", "/path/foo", False),
            ("g4", "repo2", "/path", None,        False),
            ("g5", "repo2", "/",     None,        True),
        )]

        class container(Container):
            def isDirectory(this): return True
        tests = [(expected, container(
            repos = Container(path = repos), path = path,
        ))
        # new tests go here:
        # -------------------
        for  expected,           repos,   path in (
            (['g1'],             'repo1', '/'),
            (['g2'],             'repo1', '/path/foo'),
            (['g2', 'g3'],       'repo1', '/path'),
            (['g1'],             'repo1', '/otherpath'),
            (['g1', 'g5'],       'repo2', '/otherpath'),
            (['g2', 'g3', 'g4'], 'repo2', '/path/blub'),
            (['g2', 'g4'],       'repo2', '/path/foo/bar'),
        )]

        class myMain(self._main.Main):
            def __init__(this):
                this._settings = Container(
                    _charset_ = 'utf-8', groups = config,
                    runtime = Container(author = 'foo')
                )

        main = myMain()
        for expected, change in tests:
            result = [group._name for group in main._getGroupsByChange(change)]
            self.failUnlessEqual(
                expected, result,
                "Did not get the expected groups (%r), but %r" %
                (expected, result)
            )

    def test02getGroupSets(self):
        """ getGroupSets returns the proper lists """
        import re
        class container(Container):
            def __eq__(this, other):
                return this.IDX == other.IDX
            def __ne__(this, other):
                return this.IDX != other.IDX
        config = [container(
            _name = name, IDX= idx, for_repos = None,
            ignore_if_other_matches = ignore,
            for_paths = paths and re.compile(paths) or None,
            exclude_paths = excl and re.compile(excl) or None,
            _sub_ = lambda x, y: None,
        )
        # put new groups here:
        # ---------------------
        for  name, paths,   excl,            ignore, idx in (
            ("g1", "/",     None,            True,     1),
            ("g2", "/path", None,            False,    3),
            ("g3", "/path", "/path/foo",     False,    2),
            ("g4", "/path", "/path/foo/bar", False,    3),
            ("g5", "/",     "/otherpath",    True,     1),
        )]

        class ch_container(Container):
            def isDirectory(this): return True
        repos = Container(path = '')
        tests = [(expected, [
            ch_container(repos = repos, path = path) for path in _set
        ])
        # new tests go here
        # -----------------
        for expected, _set in (
            (
                [ (['g1', 'g5'], ['/']), ],
                ('/')
            ),
            (
                [
                    (['g2'], ['/path', '/path/foo/bar']),
                    (['g3'], ['/path']),
                    (['g4'], ['/path']),
                ],
                ('/path', '/path/foo/bar')
            ),
            (
                [
                    (['g1'], ['/', '/otherpath']),
                    (['g2', 'g4'], ['/path', '/path/foo']),
                    (['g3'], ['/path']),
                    (['g5'], ['/']),
                ],
                ('/', '/path/foo', '/path', '/otherpath',)
            ),
        )]

        class myMain(self._main.Main):
            def __init__(this, changes):
                this._settings = Container(
                    _charset_ = 'utf-8', groups = config,
                    runtime = Container(author = 'foo'),
                )
                this.__changes = changes
            def _getChanges(this):
                return this.__changes

        for expected, changes in tests:
            main = myMain(changes)
            result = [(
                [group._name for group in groupset.groups],
                [change.path for change in groupset.changes],
            ) for groupset in main._getGroupSets()]
            for groups, paths in result:
                groups.sort(); paths.sort()
            result.sort()
            self.failUnlessEqual(
                expected, result,
                "Did not get the expected groupset (%r), but %r" %
                (expected, result)
            )

_tests.append(MainTestCase)


class Container(object):
    def __init__(this, **kwargs):
        this._dict = kwargs
        this._subst = {}
    def __getattr__(this, name):
        return this._dict.get(name)
    def __setitem__(this, name, value):
        this._subst[name] = value


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
