# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,E0213,W0613,W0232,W0706,R0912,F0203,W0612
# pylint: disable-msg=R0904,W0302
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.settings.configfile

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for settings.configfile tests """
    LOAD_MODULES = (('_config', 'svnmailer.settings.configfile'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_util(),
            self._mock_svnmailer_settings_base(),
            self._mock_svnmailer_settings_fileparser(),
            self._mock_errno(),
            self._mock_os(),
        ]


#### Test Cases                                     ####
########################################################
class ErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.Error """

    def test00general(self):
        """ Error is a public _base.Error """
        expected = self.mock['svnmailer.settings._base'].Error
        result = self._config.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ErrorTestCase)


class ConfigIOErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigIOError """

    def test00general(self):
        """ ConfigIOError is a public Error """
        expected = self._config.Error
        result = self._config.ConfigIOError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigIOErrorTestCase)


class ConfigMissingErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigMissingError """

    def test00general(self):
        """ ConfigMissingError is a public Error """
        expected = self._config.Error
        result = self._config.ConfigMissingError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigMissingErrorTestCase)


class ConfigInvalidErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigInvalidError """

    def test00general(self):
        """ ConfigInvalidError is a public Error """
        expected = self._config.Error
        result = self._config.ConfigInvalidError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigInvalidErrorTestCase)


class ConfigMappingSpecInvalidErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigMappingSpecInvalidError """

    def test00general(self):
        """ ConfigMappingSpecInvalidError is a public ConfigInvalidError """
        expected = self._config.ConfigInvalidError
        result = self._config.ConfigMappingSpecInvalidError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigMappingSpecInvalidErrorTestCase)


class ConfigSectionNotFoundErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigSectionNotFoundError """

    def test00general(self):
        """ ConfigSectionNotFoundError is a public ConfigInvalidError """
        expected = self._config.ConfigInvalidError
        result = self._config.ConfigSectionNotFoundError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigSectionNotFoundErrorTestCase)


class ConfigOptionUnknownErrorTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigOptionUnknownError """

    def test00general(self):
        """ ConfigOptionUnknownError is a public ConfigInvalidError """
        expected = self._config.ConfigInvalidError
        result = self._config.ConfigOptionUnknownError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

_tests.append(ConfigOptionUnknownErrorTestCase)


class ConfigFileSettingsTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigFileSettings """
    PREFIX = "ConfigFileSettings"

    def setUp(self):
        """ Extends the test environment """
        super(ConfigFileSettingsTestCase, self).setUp()
        class MyConfig(object):
            def __init__(this):
                this._section = [('name', 'value'), ('name2', 'value2')]
            def extractSection(this, section):
                if self.mock.error == 'nomaps':
                    raise self._config.ConfigSectionNotFoundError()
                for sect in this._section:
                    yield sect

        class MyFinder(object):
            def __init__(this, config):
                pass
            def find(this, spec):
                if 'value' in spec:
                    return 'mapper.%s' % spec
                return None
            def cleanup(this):
                self.mock.result.append('cleanup')

        class MySettings0(self._config.ConfigFileSettings):
            def _createFileConfig(this):
                if self.mock.error == 'parse':
                    raise this._PARSEERROR()
                self.mock.result.append('_createFileConfig')
                return base.Store(charset = 'somecharset', cfg = '*config')
            def _extractMapSections(this, config):
                self.mock.result.extend(['_extractMapSections', config.cfg])
                return '*maps'
            def _extractGeneralSection(this, config):
                if self.mock.error == 'value':
                    raise ValueError('foooo')
                self.mock.result.extend(['_extractGeneralSection', config.cfg])
                return '*general'
            def _extractGroupSections(this, config):
                if self.mock.error == 'type':
                    raise TypeError('baaar')
                self.mock.result.extend(['_extractGroupSections', config.cfg])
                return '*groups'

        class MySettings1(self._config.ConfigFileSettings):
            _charset = 'us-ascii'
            MAPSECTION = u'foo'
            DEFAULTSECTION = u'bar'

            def _createGroupContainer(this, **initkw):
                self.mock.result.extend(['_createGroupContainer', initkw])
                return '*config.%s' % initkw.get('_name')
            def _createDefaultGroupContainer(this, **initkw):
                self.mock.result.extend(['_createDefaultGroupContainer',
                    initkw])
                class Container(str):
                    for_repos = 'default-for-repos'
                    for_paths = 'default-for-paths'
                    def __call__(this, arg):
                        if arg == 'values':
                            return {'foo': 'bar'}
                        raise KeyError()
                return Container('*config.default')
            def _createGeneralContainer(this, **initkw):
                self.mock.result.extend(['_createGeneralContainer', initkw])
                return '*config.general'
            def _createMapFinder(this, config):
                return MyFinder(config)

        class MySettings2(MySettings1):
            def _passConfig(this, config, container, section):
                if self.mock.error == 'nopasssection':
                    raise self._config.ConfigSectionNotFoundError()
                self.mock.result.extend(['_passConfig',
                    config, container, section])
                return container

        class MySettings3(MySettings2):
            def _getGroupDefaults(this, config):
                def defaults(arg):
                    if arg == 'values':
                        return {'foo': 'bar'}
                return defaults, defaults('values')

        self._myconfig = MyConfig
        self._mysettings0 = MySettings0
        self._mysettings1 = MySettings1
        self._mysettings2 = MySettings2
        self._mysettings3 = MySettings3

    def test00general(self):
        """ ConfigFileSettings is a public _base.BaseSettings """
        expected = self.mock['svnmailer.settings._base'].BaseSettings
        result = self._config.ConfigFileSettings

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._config),
            "Is not public")

    def test01initial(self):
        """ ._PARSEERROR is properly filled initially """
        cls = self._config.ConfigFileSettings
        self.mock.result.append(cls._PARSEERROR)

        self.expect(self.mock['svnmailer.settings._fileparser'].Error)

    def test02init(self):
        """ .init initializes instance attributes """
        inst = self._mysettings0()
        inst.init()
        self.mock.result.extend([
            inst._charset, inst._maps, inst.general, inst.groups
        ])

        self.expect(
            '_createFileConfig',
            '_extractMapSections', '*config',
            '_extractGeneralSection', '*config',
            '_extractGroupSections', '*config',
            'somecharset', '*maps', '*general', '*groups'
        )

    def test03init(self):
        """ .init converts ValueError """
        self.mock.error = 'value'
        inst = self._mysettings0()

        self.failUnlessRaises(self._config.ConfigInvalidError, inst.init)
        self.expect(
            '_createFileConfig',
            '_extractMapSections', '*config',
            '_base.Error', ('foooo',), {}
        )

    def test04init(self):
        """ .init converts TypeError """
        self.mock.error = 'type'
        inst = self._mysettings0()

        self.failUnlessRaises(self._config.ConfigInvalidError, inst.init)
        self.expect(
            '_createFileConfig',
            '_extractMapSections', '*config',
            '_extractGeneralSection', '*config',
            '_base.Error', ('baaar',), {}
        )

    def test05init(self):
        """ .init converts ._PARSEERROR """
        self.mock.error = 'parse'
        inst = self._mysettings0()
        class MyError(Exception):
            def __init__(this):
                Exception.__init__(this)
                self.mock.result.append('MyError')
        inst._PARSEERROR = MyError

        self.failUnlessRaises(self._config.ConfigInvalidError, inst.init)
        self.expect('MyError', '_base.Error', ('',), {})

    def test06extractGroupSections(self):
        """ ._extractGroupSections returns the group configs """
        inst = self._mysettings3()
        self.mock.result.append(inst._extractGroupSections(['alpha', 'beta']))

        self.expect(
            '_createGroupContainer', {'foo': 'bar', '_name': 'alpha'},
            '_passConfig', ['alpha', 'beta'], '*config.alpha', 'alpha',
            '_createGroupContainer', {'foo': 'bar', '_name': 'beta'},
            '_passConfig', ['alpha', 'beta'], '*config.beta', 'beta',
            ['*config.alpha', '*config.beta'],
        )

    def test07extractGroupSections(self):
        """ ._extractGroupSections returns the default config only """
        inst = self._mysettings3()
        self.mock.result.append(inst._extractGroupSections([]))

        self.expect('_createGroupContainer', {'foo': 'bar'}, ['*config.None'])

    def test08extractGeneralSection(self):
        """ ._extractGeneralSection returns the general config """
        inst = self._mysettings3()
        inst._charset = 'iso-8859-15'
        inst.GENERALSECTION = u'\u20acuro'
        self.mock.result.append(inst._extractGeneralSection('*input'))

        self.expect(
            '_createGeneralContainer', {},
            '_passConfig', '*input', '*config.general', '\xa4uro',
            '*config.general'
        )

    def test09extractMapSections(self):
        """ ._extractMapSections returns the maps """
        config = self._myconfig()
        inst = self._mysettings3()
        inst._charset = 'iso-8859-15'
        inst.MAPSECTION = u'\u20acuro'
        self.mock.result.append(inst._extractMapSections(config))

        self.expect(
            'cleanup',
            {'name2': 'mapper.value2', 'name': 'mapper.value'}
        )

    def test10extractMapSections(self):
        """ ._extractMapSections returns empty maps on None MAPSECTION """
        inst = self._mysettings3()
        inst.MAPSECTION = None
        self.mock.result.append(inst._extractMapSections('*config'))

        self.expect({})

    def test11extractMapSections(self):
        """ ._extractMapSections returns empty maps on missing MAPSECTION """
        self.mock.error = 'nomaps'
        config = self._myconfig()
        inst = self._mysettings3()
        self.mock.result.append(inst._extractMapSections(config))

        self.expect('cleanup', {})

    def test12extractMapSections(self):
        """ ._extractMapSections raises ConfigMappingSpecInvalidError """
        config = self._myconfig()
        config._section = [('foo', 'bar')]
        inst = self._mysettings3()

        self.failUnlessRaises(self._config.ConfigMappingSpecInvalidError,
            inst._extractMapSections, config)

    def test13getGroupDefaults(self):
        """ ._getGroupDefaults returns the default group """
        inst = self._mysettings2()
        inst._charset = 'iso-8859-15'
        inst.DEFAULTSECTION = u'\u20acuro'
        inst.general = base.Store(diff_command = 'diffcmd')
        self.mock.result.append(inst._getGroupDefaults('*config'))

        self.expect(
            '_createDefaultGroupContainer', {'diff_command': 'diffcmd',
                '_name': u'\u20acuro'},
            '_passConfig', '*config', '*config.default', '\xa4uro',
            ('*config.default', {'foo': 'bar',
                '_def_for_repos': 'default-for-repos',
                '_def_for_paths': 'default-for-paths'})
        )

    def test14getGroupDefaults(self):
        """ ._getGroupDefaults deals with missing [defaults] """
        self.mock.error = 'nopasssection'
        inst = self._mysettings2()
        inst.general = base.Store(diff_command = 'diffcmd')
        self.mock.result.append(inst._getGroupDefaults('*config1'))

        self.expect(
            '_createDefaultGroupContainer', {'diff_command': 'diffcmd',
                '_name': u'bar'},
            ('*config.default', {'foo': 'bar', '_def_for_repos':
                'default-for-repos', '_def_for_paths': 'default-for-paths'})
        )

    def test15passConfig(self):
        """ ._passConfig passes the section to the container """
        config = self._myconfig()
        inst = self._mysettings1()
        container = base.Store()
        res = inst._passConfig(config, container, 'foo')
        self.mock.result.extend([res.name, res.name2])

        self.failUnless(res is container, "Not identical")
        self.expect('value', 'value2')

    def test16passConfig(self):
        """ ._passConfig converts AttributeError """
        config = self._myconfig()
        inst = self._mysettings1()
        class Container(object):
            def raiser(this, value):
                raise AttributeError()
            name = property(fset = raiser)
        container = Container()

        self.failUnlessRaises(self._config.ConfigOptionUnknownError,
            inst._passConfig, config, container, 'foo')

    def test17passConfig(self):
        """ ._passConfig passes ConfigSectionNotFoundError """
        self.mock.error = 'nomaps'
        config = self._myconfig()
        inst = self._mysettings1()
        container = base.Store()

        self.failUnlessRaises(self._config.ConfigSectionNotFoundError,
            inst._passConfig, config, container, 'foo')

    def test18createFileConfig(self):
        """ ._createFileConfig returns a new FileConfig instance """
        class FileConfig(object):
            def __new__(cls, settingsobj):
                self.mock.result.append(settingsobj)
                return '*fileconfig'
        self._config.FileConfig = FileConfig
        inst = self._mysettings1()
        self.mock.result.append(inst._createFileConfig())

        self.expect(inst, '*fileconfig')

_tests.append(ConfigFileSettingsTestCase)


class FileConfigTestCase(TestBase):
    """ svnmailer.settings.configfile.ConfigFile """
    PREFIX = "FileConfig"

    def setUp(self):
        """ Extends the test environment """
        super(FileConfigTestCase, self).setUp()
        class MySection(list):
            name = 'foo'
            def __delitem__(this, name):
                this[:] = [item for item in this if item.name != name]

        def myoption(name, value):
            return base.Store(name = name, value = value)

        class MyParser(object):
            def __init__(this):
                this.__mocked__ = '*MyParser'
                this.contains = []
            def slurp(this, fp, name):
                if self.mock.error == 'ioerror':
                    raise IOError()
                self.mock.result.extend(['slurp', fp.__mocked__, name])
            def __contains__(this, value):
                return value in this.contains

        class MyStream(object):
            def __init__(this):
                this.name = 'somefile'
                this.__mocked__ = '*MyStream'
            def close(this):
                self.mock.result.append('close')

        class MySettings(object):
            GENERALSECTION = u'plop'

        class MyConfig(self._config.FileConfig):
            def __init__(this):
                this._parsed = {}
                this.charset = 'us-ascii'

        class MyConfig1(MyConfig):
            _CHARSETOPTION = 'foo'
            _INCLUDEOPTION = 'bar'
            _options = {'foo': base.Store(value = 'gnar')}

            def _createFileParser(this):
                return MyParser()
            def _findConfig(this, settingsobj):
                if self.mock.error == 'missing':
                    raise self._config.ConfigMissingError()
                return MyStream()
            def _extractOption(this, section, name):
                self.mock.result.extend(['_extractOption', section, name])
                return this._options.get(name)
            def _attachIncludes(this, parsed, settingsobj, includes, name):
                if self.mock.error == 'ioerror':
                    raise self._config.ConfigIOError()
                elif self.mock.error == 'value':
                    raise ValueError()
                self.mock.result.extend(['_attachIncludes',
                    parsed, settingsobj, includes, name])

        class MyConfig2(MyConfig1):
            def _processPreOptions(this, parser, settingsobj, name):
                if self.mock.error == 'value':
                    raise ValueError()
                self.mock.result.extend(['_processPreOptions',
                    parser.__mocked__, settingsobj, name])

        class MyConfig3(MyConfig):
            def _optionxform(this, name):
                return "transformed(%s)" % name
            def _getDefaultConfigFiles(this, settingsobj):
                return ['foo', 'bar']

        self._myconfig = MyConfig
        self._myconfig1 = MyConfig1
        self._myconfig2 = MyConfig2
        self._myconfig3 = MyConfig3
        self._mysection = MySection
        self._myoption = myoption
        self._mysettings = MySettings
        self._mystream = MyStream
        self._myparser = MyParser

    def test00general(self):
        """ FileConfig is a private _base.BaseConfig """
        expected = self.mock['svnmailer.settings._base'].BaseConfig
        result = self._config.FileConfig

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failIf(result.__name__ in self.public(self._config),
            "Is public")

    def test01initial(self):
        """ FileConfig's class attributes are properly filled initially """
        cls = self._config.FileConfig
        self.mock.result.extend([cls._CHARSETOPTION, cls._INCLUDEOPTION])

        self.expect('config_charset', 'include_config')

    def test02init(self):
        """ .__init__ initializes super class and instance attributes """
        class MyConfig(self._config.FileConfig):
            def _loadConfig(this, settingsobj):
                return "loaded.%s" % settingsobj
        inst = MyConfig('*settings')
        self.mock.result.append(inst._parsed)

        self.expect('BaseConfig', '*settings', 'loaded.*settings')

    def test03init(self):
        """ .__init__ passes ConfigIOError """
        class MyConfig(self._config.FileConfig):
            def _loadConfig(this, settingsobj):
                raise self._config.ConfigIOError()

        self.failUnlessRaises(self._config.ConfigIOError, MyConfig, 'foo')

    def test04init(self):
        """ .__init__ passes ConfigMissingError """
        class MyConfig(self._config.FileConfig):
            def _loadConfig(this, settingsobj):
                raise self._config.ConfigMissingError()

        self.failUnlessRaises(self._config.ConfigMissingError, MyConfig, 'foo')

    def test05init(self):
        """ .__init__ passes ValueError """
        class MyConfig(self._config.FileConfig):
            def _loadConfig(this, settingsobj):
                raise ValueError()

        self.failUnlessRaises(ValueError, MyConfig, 'foo')

    def test06init(self):
        """ .__init__ passes ConfigInvalidError """
        class MyConfig(self._config.FileConfig):
            def _loadConfig(this, settingsobj):
                raise self._config.ConfigInvalidError()

        self.failUnlessRaises(self._config.ConfigInvalidError, MyConfig, 'foo')

    def test07iter(self):
        """ .__iter__ returns a section name iterator """
        inst = self._myconfig()
        inst._parsed = [base.Store(name = 'n1'), base.Store(name = 'n2')]
        for name in inst:
            self.mock.result.append(name)

        self.expect('n1', 'n2')

    def test08delitem(self):
        """ .__delitem__ removes the specified section """
        inst = self._myconfig()
        inst._parsed = {'n1': 'foo', 'n2': 'bar'}
        del inst['n1']
        self.mock.result.append(inst._parsed)

        self.expect({'n2': 'bar'})

    def test09delitem(self):
        """ .__delitem__ raises KeyError """
        inst = self._myconfig()
        inst._parsed = {'n2': 'bar'}
        def deler(name):
            del inst[name]

        self.failUnlessRaises(KeyError, deler, 'n1')

    def test10extractSection(self):
        """ .extractSection returns a k-v iterator """
        inst = self._myconfig3()
        opt = self._myoption
        inst._parsed = {
            'foo': self._mysection([opt('x', 1), opt('y', 22)]),
            'bar': 'baz',
        }
        for name, value in inst.extractSection('foo'):
            self.mock.result.append((name, value))
        self.mock.result.append(inst._parsed)

        self.expect(
            ('transformed(x)', 1), ('transformed(y)', 22), {'bar': 'baz'}
        )

    def test11extractSection(self):
        """ .extractSection keeps the section """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('x', 1), opt('y', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        for name, value in inst.extractSection('foo', keep = True):
            self.mock.result.append((name, value))
        self.mock.result.append(inst._parsed)

        self.expect(
            ('transformed(x)', 1), ('transformed(y)', 22),
            {'foo': section, 'bar': 'baz'}
        )

    def test12extractSection(self):
        """ .extractSection ignores KeyError on section deletion """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('x', 1), opt('y', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        count = 0
        for name, value in inst.extractSection('foo'):
            self.mock.result.append((name, value))
            count += 1
            if count == 1:
                del inst._parsed['foo']
        self.mock.result.append(inst._parsed)

        self.expect(
            ('transformed(x)', 1), ('transformed(y)', 22), {'bar': 'baz'}
        )

    def test13extractSection(self):
        """ .extractSection checks the option names """
        class MyConfig(self._myconfig):
            def _optionxform(this, name):
                return name
        inst = MyConfig()
        opt = self._myoption
        section = self._mysection([opt('x', 1), opt('_y', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        try:
            for name, value in inst.extractSection('foo'):
                self.mock.result.append((name, value))
            self.fail("Doesn't raise ConfigOptionUnknownError")
        except self._config.ConfigOptionUnknownError:
            """ fine """
            pass

    def test14extractSection(self):
        """ .extractSection doesn't check the option names """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('x', 1), opt('_y', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        for name, value in inst.extractSection('foo', check = False):
            self.mock.result.append((name, value))
        self.mock.result.append(inst._parsed)

        self.expect(
            ('transformed(x)', 1), ('transformed(_y)', 22), {'bar': 'baz'}
        )

    def test15extractSection(self):
        """ .extractSection transforms option names """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('x-y', 1), opt('zb-a', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        for name, value in inst.extractSection('foo'):
            self.mock.result.append((name, value))
        self.mock.result.append(inst._parsed)

        self.expect(
            ('transformed(x-y)', 1), ('transformed(zb-a)', 22), {'bar': 'baz'}
        )

    def test16extractSection(self):
        """ .extractSection doesn't transform option names """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('x-y', 1), opt('zb-a', 22)])
        inst._parsed = {'foo': section, 'bar': 'baz'}
        for name, value in inst.extractSection('foo', xform = False):
            self.mock.result.append((name, value))
        self.mock.result.append(inst._parsed)

        self.expect(('x-y', 1), ('zb-a', 22), {'bar': 'baz'})

    def test17extractSection(self):
        """ .extractSection raises ConfigSectionNotFoundError """
        inst = self._myconfig3()
        inst._parsed = {'bar': 'baz'}

        try:
            for name, value in inst.extractSection('foo'):
                pass
            self.fail("Doesn't raise ConfigSectionNotFoundError")
        except self._config.ConfigSectionNotFoundError:
            """ fine """
            pass

    def test18optionxform(self):
        """ ._optionxform transforms options properly """
        inst = self._myconfig()
        self.mock.result.append(inst._optionxform('A-B+c'))

        self.expect('a_b+c')

    def test19loadConfig(self):
        """ ._loadConfig returns the parsed config """
        inst = self._myconfig2()
        self.mock.result.append(inst._loadConfig('*settings').__mocked__)

        self.expect(
            'slurp', '*MyStream', 'somefile',
            'close',
            '_processPreOptions', '*MyParser', '*settings', 'somefile',
            '*MyParser'
        )

    def test20loadConfig(self):
        """ ._loadConfig converts IOError """
        self.mock.error = "ioerror"
        inst = self._myconfig2()

        self.failUnlessRaises(self._config.ConfigIOError,
            inst._loadConfig, '*settings')

    def test21loadConfig(self):
        """ ._loadConfig passes ConfigMissingError """
        self.mock.error = "missing"
        inst = self._myconfig2()

        self.failUnlessRaises(self._config.ConfigMissingError,
            inst._loadConfig, '*settings')

    def test22loadConfig(self):
        """ ._loadConfig passes ValueError """
        self.mock.error = "value"
        inst = self._myconfig2()

        self.failUnlessRaises(ValueError, inst._loadConfig, '*settings')

    def test23processPreOptions(self):
        """ ._processPreOptions processes charset from pre options """
        inst = self._myconfig1()
        parsed = {None: self._mysection(['a', 'b'])}
        inst._processPreOptions(parsed, '*settings', 'somename')
        self.mock.result.append(inst.charset)

        self.expect(
            '_extractOption', ['a', 'b'], 'foo',
            '_extractOption', ['a', 'b'], 'bar',
            'gnar'
        )

    def test24processPreOptions(self):
        """ ._processPreOptions processes [general] as fallback """
        inst = self._myconfig1()
        settings = base.Store(GENERALSECTION = u'boo')
        parsed = {'boo': self._mysection(['a', 'b'])}
        inst._processPreOptions(parsed, settings, 'somename')
        self.mock.result.append(inst.charset)

        self.expect(
            '_extractOption', ['a', 'b'], 'foo',
            '_extractOption', ['a', 'b'], 'bar',
            'gnar'
        )

    def test25processPreOptions(self):
        """ ._processPreOptions deals with missing [general] """
        inst = self._myconfig1()
        settings = base.Store(GENERALSECTION = u'boo')
        parsed = {}
        inst._processPreOptions(parsed, settings, 'somename')

        self.expect()

    def test26processPreOptions(self):
        """ ._processPreOptions processes includes """
        inst = self._myconfig1()
        parsed = {None: self._mysection(['a', 'b'])}
        inst._options['bar'] = base.Store(value = 'someincludes')
        inst._processPreOptions(parsed, '*settings', 'somename')

        self.expect(
            '_extractOption', ['a', 'b'], 'foo',
            '_extractOption', ['a', 'b'], 'bar',
            '_attachIncludes', {None: ['a', 'b']}, '*settings',
                'someincludes', 'somename'
        )

    def test27processPreOptions(self):
        """ ._processPreOptions deletes empty presection """
        inst = self._myconfig1()
        section = self._mysection()
        section.name = None
        parsed = {None: section}
        inst._options['bar'] = base.Store(value = 'someincludes')
        inst._processPreOptions(parsed, '*settings', 'somename')
        self.mock.result.append(parsed)

        self.expect(
            '_extractOption', [], 'foo',
            '_extractOption', [], 'bar',
            '_attachIncludes', {}, '*settings', 'someincludes', 'somename',
            {}
        )

    def test28processPreOptions(self):
        """ ._processPreOptions raises ConfigInvalidError """
        inst = self._myconfig1()
        opt = self._myoption
        section = self._mysection([opt('a', 1)])
        section.name = None
        parsed = {None: section}
        inst._options['bar'] = base.Store(value = 'someincludes')

        self.failUnlessRaises(self._config.ConfigInvalidError,
            inst._processPreOptions, parsed, '*settings', 'somename')

    def test29processPreOptions(self):
        """ ._processPreOptions passes ConfigIOError """
        self.mock.error = 'ioerror'
        inst = self._myconfig1()
        opt = self._myoption
        section = self._mysection([])
        section.name = None
        parsed = {None: section}
        inst._options['bar'] = base.Store(value = 'someincludes')

        self.failUnlessRaises(self._config.ConfigIOError,
            inst._processPreOptions, parsed, '*settings', 'somename')

    def test30processPreOptions(self):
        """ ._processPreOptions passes ValueError """
        self.mock.error = 'value'
        inst = self._myconfig1()
        opt = self._myoption
        section = self._mysection([])
        section.name = None
        parsed = {None: section}
        inst._options['bar'] = base.Store(value = 'someincludes')

        self.failUnlessRaises(ValueError,
            inst._processPreOptions, parsed, '*settings', 'somename')

    def test31attachIncludes(self):
        """ ._attachIncludes loads the includes """
        inst = self._myconfig()
        parsed = self._myparser()
        settings = base.Store(runtime = base.Store(path_encoding = 'penc'))
        includes = 'a b'
        mainfile = 'to/file'
        self.mock.os.path.abspath['to'] = '/some/path/to/file'
        self.mock.os.path.dirname['to/file'] = 'to'
        def newfile(name, mode = 'r'):
            self.mock.result.extend(['file', name, mode])
            return self._mystream()
        self._config.file = newfile
        inst._attachIncludes(parsed, settings, includes, mainfile)

        self.expect(
            'os.path.dirname', 'to/file',
            'os.path.abspath', 'to',
            'toLocale', 'a', 'us-ascii', 'penc', False,
            'toLocale', 'b', 'us-ascii', 'penc', False,
            'os.path.join', ('/some/path/to/file', 'locale(a, penc)'),
            'file', '/some/path/to/file/locale(a, penc)', 'r',
            'slurp', '*MyStream', 'somefile',
            'close',
            'os.path.join', ('/some/path/to/file', 'locale(b, penc)'),
            'file', '/some/path/to/file/locale(b, penc)', 'r',
            'slurp', '*MyStream', 'somefile',
            'close'
        )

    def test32attachIncludes(self):
        """ ._attachIncludes converts IOError """
        inst = self._myconfig()
        parsed = self._myparser()
        settings = base.Store(runtime = base.Store(path_encoding = 'penc'))
        includes = 'a b'
        mainfile = 'to/file'
        self.mock.os.path.abspath['to'] = '/some/path/to/file'
        self.mock.os.path.dirname['to/file'] = 'to'
        def newfile(name, mode = 'r'):
            raise IOError()
        self._config.file = newfile
        self.failUnlessRaises(self._config.ConfigIOError,
            inst._attachIncludes, parsed, settings, includes, mainfile)

    def test33attachIncludes(self):
        """ ._attachIncludes raises ConfigInvalidError """
        inst = self._myconfig()
        parsed = self._myparser()
        parsed.contains.append(None)
        settings = base.Store(runtime = base.Store(path_encoding = 'penc'))
        includes = 'a b'
        mainfile = 'to/file'
        self.mock.os.path.abspath['to'] = '/some/path/to/file'
        self.mock.os.path.dirname['to/file'] = 'to'
        def newfile(name, mode = 'r'):
            return self._mystream()
        self._config.file = newfile
        self.failUnlessRaises(self._config.ConfigInvalidError,
            inst._attachIncludes, parsed, settings, includes, mainfile)

    def test34attachIncludes(self):
        """ ._attachIncludes passes ValueError """
        self.mock.error = 'util.filename'
        inst = self._myconfig()
        parsed = self._myparser()
        settings = base.Store(runtime = base.Store(path_encoding = 'penc'))
        includes = 'a b'
        mainfile = 'to/file'
        self.mock.os.path.abspath['to'] = '/some/path/to/file'
        self.mock.os.path.dirname['to/file'] = 'to'
        def newfile(name, mode = 'r'):
            return self._mystream()
        self._config.file = newfile
        self.failUnlessRaises(ValueError,
            inst._attachIncludes, parsed, settings, includes, mainfile)

    def test35extractOption(self):
        """ ._extractOption returns and deletes the option """
        inst = self._myconfig3()
        opt = self._myoption
        section = self._mysection([opt('n1', 'v1'), opt('n2', 'v2')])
        res = inst._extractOption(section, 'transformed(n1)')
        self.mock.result.append((res.name, res.value))
        res = inst._extractOption(section, 'n1')
        self.mock.result.append(res)
        self.mock.result.append([(item.name, item.value) for item in section])

        self.expect(('n1', 'v1'), None, [('n2', 'v2')])

    def test36findConfig(self):
        """ ._findConfig opens the runtime config file """
        settings = base.Store(runtime = base.Store(
            config = 'somefile', path_encoding = 'someencoding',
        ))
        def newfile(name, mode = 'r'):
            return "file:%s,%s" % (name, mode)
        self._config.file = newfile
        inst = self._myconfig()
        self.mock.os.path.abspath['locale(somefile, someencoding)'] = \
            '/path/to/locale(somefile, someencoding)'
        self.mock.result.append(inst._findConfig(settings))

        self.expect(
            'toLocale', 'somefile', 'us-ascii', 'someencoding', False,
            'os.path.abspath', 'locale(somefile, someencoding)',
            'file:/path/to/locale(somefile, someencoding),r'
        )

    def test37findConfig(self):
        """ ._findConfig converts IOError on runtime config """
        settings = base.Store(runtime = base.Store(
            config = 'somefile', path_encoding = 'someencoding',
        ))
        def newfile(name, mode = 'r'):
            raise IOError()
        self._config.file = newfile
        inst = self._myconfig()
        self.mock.os.path.abspath['locale(somefile, someencoding)'] = \
            '/path/to/locale(somefile, someencoding)'

        self.failUnlessRaises(self._config.ConfigIOError,
            inst._findConfig, settings)

    def test38findConfig(self):
        """ ._findConfig tries the default config files """
        settings = base.Store(runtime = base.Store(config = None))
        def newfile(name, mode = 'r'):
            if name == 'foo':
                raise IOError(self.mock['errno'].ENOENT, 'plop')
            return 'file:%s,%s' % (name, mode)
        self._config.file = newfile
        inst = self._myconfig3()
        self.mock.result.append(inst._findConfig(settings))

        self.expect('file:bar,r')

    def test39findConfig(self):
        """ ._findConfig converts IOError """
        settings = base.Store(runtime = base.Store(config = None))
        def newfile(name, mode = 'r'):
            if name == 'foo':
                raise IOError(self.mock['errno'].ENOENT, 'plop')
            elif name == 'bar':
                raise IOError(self.mock['errno'].ENOMEM, 'plop')
        self._config.file = newfile
        inst = self._myconfig3()

        self.failUnlessRaises(self._config.ConfigIOError,
            inst._findConfig, settings)

    def test40findConfig(self):
        """ ._findConfig raises ConfigMissingError """
        settings = base.Store(runtime = base.Store(config = None))
        def newfile(name, mode = 'r'):
            raise IOError(self.mock['errno'].ENOENT, 'plop')
        self._config.file = newfile
        inst = self._myconfig3()

        self.failUnlessRaises(self._config.ConfigMissingError,
            inst._findConfig, settings)

    def test41getDefaultConfigFiles(self):
        """ ._getDefaultConfigFiles returns unicode files """
        import sys
        settings = base.Store(runtime = base.Store(
            path_encoding = 'someencoding', repository = 'somerepos',
        ))
        inst = self._myconfig()
        self.mock.os.path.dirname[u'unicode(somescript, someencoding)'] = \
            u'foo'
        self.mock.os.path.abspath.update({
            u'locale(somerepos/conf/mailer.conf, someencoding)':
                u'/plop/locale(somerepos/conf/mailer.conf, someencoding)',
            u'locale(foo/mailer.conf, someencoding)':
                u'/blubb/locale(foo/mailer.conf, someencoding)',
            u'locale(//etc/svn-mailer.conf, someencoding)':
                u'locale(//etc/svn-mailer.conf, someencoding)',
        })
        old_argv = sys.argv
        try:
            sys.argv = [u'somescript']
            self.mock.result.append(inst._getDefaultConfigFiles(settings))
        finally:
            sys.argv = old_argv

        self.expect(
            'fromLocale', u'somescript', 'someencoding',
            'os.path.dirname', u'unicode(somescript, someencoding)',
            'os.path.join', ('somerepos', u'conf', u'mailer.conf'),
            'os.path.join', (u'foo', u'mailer.conf'),
            'os.path.join', (u'/', u'etc', u'svn-mailer.conf'),
            'toLocale', u'somerepos/conf/mailer.conf', None, 'someencoding',
                False,
            'toLocale', u'foo/mailer.conf', None, 'someencoding', False,
            'toLocale', u'//etc/svn-mailer.conf', None, 'someencoding', False,
            'os.path.abspath', u'locale(somerepos/conf/mailer.conf, '
                'someencoding)',
            'os.path.abspath', u'locale(foo/mailer.conf, someencoding)',
            'os.path.abspath', u'locale(//etc/svn-mailer.conf, someencoding)',
            [u'/plop/locale(somerepos/conf/mailer.conf, someencoding)',
                u'/blubb/locale(foo/mailer.conf, someencoding)',
                u'locale(//etc/svn-mailer.conf, someencoding)']
        )

    def test42getDefaultConfigFiles(self):
        """ ._getDefaultConfigFiles returns string files """
        import sys
        settings = base.Store(runtime = base.Store(
            path_encoding = 'someencoding', repository = 'somerepos',
        ))
        inst = self._myconfig()
        self.mock.os.path.dirname['unicode(somescript, someencoding)'] = 'foo'
        self.mock.os.path.abspath.update({
            'somerepos/conf/mailer.conf': '/plopp/somerepos/conf/mailer.conf',
            'foo/mailer.conf': '/blubb/foo/mailer.conf',
            '//etc/svn-mailer.conf': '//etc/svn-mailer.conf',
        })
        old_argv = sys.argv
        try:
            sys.argv = ['somescript']
            self.mock.result.append(inst._getDefaultConfigFiles(settings))
        finally:
            sys.argv = old_argv

        self.expect(
            'fromLocale', 'somescript', 'someencoding',
            'os.path.dirname', 'unicode(somescript, someencoding)',
            'os.path.join', ('somerepos', 'conf', 'mailer.conf'),
            'os.path.join', ('foo', 'mailer.conf'),
            'os.path.join', ('/', 'etc', 'svn-mailer.conf'),
            'os.path.abspath', 'somerepos/conf/mailer.conf',
            'os.path.abspath', 'foo/mailer.conf',
            'os.path.abspath', '//etc/svn-mailer.conf',
            ['/plopp/somerepos/conf/mailer.conf', '/blubb/foo/mailer.conf',
                '//etc/svn-mailer.conf']
        )

    def test43createFileParser(self):
        """ ._createFileParser returns a new FileParser instance """
        class MyParser(object):
            def __new__(cls):
                return '*fileparser'
        self.mock['svnmailer.settings._fileparser'].FileParser = MyParser
        inst = self._myconfig()
        self.mock.result.append(inst._createFileParser())

        self.expect('*fileparser')

_tests.append(FileConfigTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
